package net.wanji.utc.service.runninginfo.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.entity.CrossPhaseLightsPO;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.databus.vo.BaseCrossInfo;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.DateStyle;
import net.wanji.utc.po.hk.request.HKRequest;
import net.wanji.utc.po.hk.request.RealTimeStateInfoQueryPO;
import net.wanji.utc.po.hk.response.*;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.util.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;

/**
 * @author Kent HAN
 * @date 2022/11/21 14:07
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class HkLightsStatusServiceImpl implements HkLightsStatusService {
    private final ArtemisConfig artemisConfig;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossInfoMapper crossInfoMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossSchemeMapper crossSchemeMapper;

    public static Map<String, String> runModeParseMap = new HashMap<String, String>();

    public static Map<String, Integer> turnParseMap = new HashMap<>();

    public static Map<String, String> directionParseMap = new HashMap<>();

    static {
        // key: 万集灯组方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北
        directionParseMap.put("1", "1");
        directionParseMap.put("2", "5");
        directionParseMap.put("3", "4");
        directionParseMap.put("4", "6");
        directionParseMap.put("5", "2");
        directionParseMap.put("6", "8");
        directionParseMap.put("7", "3");
        directionParseMap.put("8", "7");
    }

    static {
        // key:海康控制模式  value:万集控制模式
        runModeParseMap.put("2", "4");   // 黄闪
        runModeParseMap.put("3", "3");   // 全红
        runModeParseMap.put("4", "11");  // 定周期=恢复时间表
        runModeParseMap.put("10", "9");  // 步进
        runModeParseMap.put("12", "2");  // 相位锁定
    }

    // key:万集车道转向：1左转；2直行；3右转；4掉头；5直左；6直右；7左直右；8左右；9左转掉头；10直行掉头；
    // 11右转掉头；12左直掉头；13直右掉头；14左直右掉头；15左右掉头
    static {
        turnParseMap.put("1", 2);
        turnParseMap.put("2", 1);
        turnParseMap.put("3", 3);
        turnParseMap.put("4", 4);
        turnParseMap.put("5", 5);
        turnParseMap.put("6", 6);
        turnParseMap.put("7", 9);
        turnParseMap.put("8", 8);
        turnParseMap.put("9", 10);
        turnParseMap.put("10", 4);
        turnParseMap.put("11", 4);
        turnParseMap.put("12", 4);
        turnParseMap.put("13", 4);
        turnParseMap.put("14", 4);
        turnParseMap.put("15", 4);
    }


    @Override
    public List<LightsStatusVO> getHkLightsStatus(String crossId) throws Exception{
        List<LightsStatusVO> result = new ArrayList<>();

        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossId(crossId);

        if (Objects.nonNull(crossInfoPO)) {
            String signalCode = crossInfoPO.getCode();
            String jsonBody = getRequestBody(signalCode);
            Object data = callHKService("getRealtimeSchemeStateInfo", jsonBody);
            if (Objects.nonNull(data)) {
                RealTimeStateInfoListPO realTimeStateInfoListPO = JSONObject.parseObject(String.valueOf(data), RealTimeStateInfoListPO.class);
                LightsStatusVO infoVo = new LightsStatusVO();
                if (!CollectionUtils.isEmpty(realTimeStateInfoListPO.getList())) {
                    for (RealTimeStateInfoPO resultPO : realTimeStateInfoListPO.getList()) {
                        infoVo = new LightsStatusVO();
                        Integer patternNo = resultPO.getPatternNo();
                        CrossSchemePO baseCrossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, patternNo);
                        List<RingsPO> rings = resultPO.getRings();                 // 环列表
                        List<ChannelStatePO> channelState = resultPO.getChannelState();// 灯组列表
                        String HKControlType = resultPO.getControlType();
                        String WJControlType = null;
                        if (StringUtils.isNotBlank(HKControlType)) {
                            WJControlType = runModeParseMap.get(HKControlType);
                        }
                        if (StringUtils.isEmpty(WJControlType)) {
                            WJControlType = "255";// 转换失败默认临时方案
                        }
                        infoVo.setRunMode(WJControlType);// 控制模式
                        infoVo.setPhasePlanId(String.valueOf(patternNo));       // 相位方案号
                        infoVo.setSchemeId(String.valueOf(patternNo));
                        int cycleLen = resultPO.getCycle();                     // 周期时长
                        int curRunTime = resultPO.getCurRunTime();              // 当前运行时长
                        int cycleCountDown = cycleLen - curRunTime;             // 周期剩余时长
                        infoVo.setCode(signalCode);
                        infoVo.setCrossId(crossId);
                        infoVo.setCycleCountDown(cycleCountDown);
                        infoVo.setCycleLen(cycleLen);
                        // 循环相位列表
                        Map<String, Object> phaseMap = new HashMap<>();
                        for (RingsPO ring : rings) {
                            List<PhaseInfoPO> phaseList = ring.getPhaseList();
                            for (PhaseInfoPO phaseInfoPO : phaseList) {
                                Integer phaseNo = phaseInfoPO.getPhaseNo();
                                Integer countdown = phaseInfoPO.getCountdown();
                                setDirTurnRedCountDown(phaseMap, crossId, crossLightsPOList, baseCrossSchemePO, phaseNo, countdown);
                                Integer vehicleStatus = phaseInfoPO.getVehicleStatus();
                                Integer phaseLength = phaseInfoPO.getPhaseLength();
                                Integer red = phaseInfoPO.getRed();
                                Integer yellow = phaseInfoPO.getYellow();
                                Integer green = phaseLength - yellow - red;
                                if (2 != vehicleStatus) { // 2红灯
                                    infoVo.setPhaseId(String.valueOf(phaseNo));
                                }

                                if (phaseInfoPO.getCountdown() <= phaseLength && !Objects.equals(phaseInfoPO.getRunTime(), phaseLength)) {
                                    infoVo.setCyclePhaseCountDown(phaseInfoPO.getCountdown());
                                }

                            }
                        }
                        infoVo.setPhaseMap(phaseMap);

                        Map<String, Object> dirLampGroupMap = new HashMap<>();
                        // 循环构建灯组对象（从灯组表获取灯组信息）
                        for (ChannelStatePO channelStatePO : channelState) {
                            Integer channelNo = channelStatePO.getChannelNo();
                            Integer state = channelStatePO.getState();
                            String color = transferHKLampGroup(state);
                            for (CrossLightsPO po : crossLightsPOList) {
                                if (StringUtils.equals(po.getLightsNo(), String.valueOf(channelNo))) {
                                    String direction = po.getDir().toString();
                                    String turn = po.getTurn().toString();
                                    if (dirLampGroupMap.get(direction) != null) {
                                        Map<Integer, String> mstr = (Map<Integer, String>) dirLampGroupMap.get(direction);
                                        mstr = setTurnColor(color, po, turn, mstr);
                                        dirLampGroupMap.put(direction, mstr);
                                    } else {
                                        Map<Integer, String> lamp = setTurnColor(color, po, turn, new HashMap<>());
                                        dirLampGroupMap.put(direction, lamp);
                                    }
                                }
                            }
                        }
                        infoVo.setDirLampGroupMap(dirLampGroupMap);
                        Date date = new Date();
                        String timeStamp = String.valueOf(Long.valueOf(date.getTime()));
                        infoVo.setTimeStamp(timeStamp);
                        LocalDateTime cycleBeginTime = resultPO.getCycleBeginTime(); // 周期开始时间
                        String schemeStartTime = DateUtil.format(cycleBeginTime, DateStyle.YYYYMMDDHHMMSS.getValue());
                        infoVo.setSchemeStartTime(schemeStartTime);
                    }
                    result.add(infoVo);
                }
            }
        }
        return result;
    }

    private Map<String, Object> setDirTurnRedCountDown(Map<String, Object> phaseMap,String crossId, List<CrossLightsPO> crossLightsPOList, CrossSchemePO crossSchemePO, Integer phaseNo, Integer countdown) {
        if (Objects.nonNull(crossSchemePO)) {
            Integer phaseId = crossPhaseMapper.selectPhaseId(crossId, crossSchemePO.getId(), phaseNo);
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByCrossId(crossId);
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                if (Objects.equals(phaseId, crossPhaseLightsPO.getPhaseId())) {
                    Integer lightsId = crossPhaseLightsPO.getLightsId();
                    for (CrossLightsPO crossLightsPO : crossLightsPOList) {
                        if (Objects.equals(lightsId, crossLightsPO.getId())) {
                            Integer dir = crossLightsPO.getDir();
                            setDirTurnCounDown(countdown, phaseMap, crossLightsPO, dir);
                        }
                    }
                }
            }
        } else { // 特殊控制的时候
            List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossId(crossId);
            for (CrossPhasePO crossPhasePO : crossPhasePOS) {
                Integer phaseId = crossPhasePO.getId();
                List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByCrossId(crossId);
                for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                    if (Objects.equals(phaseId, crossPhaseLightsPO.getPhaseId())) {
                        Integer lightsId = crossPhaseLightsPO.getLightsId();
                        for (CrossLightsPO crossLightsPO : crossLightsPOList) {
                            if (Objects.equals(lightsId, crossLightsPO.getId())) {
                                Integer dir = crossLightsPO.getDir();
                                setDirTurnCounDown(countdown, phaseMap, crossLightsPO, dir);
                            }
                        }
                    }
                }
            }
        }
        return phaseMap;
    }

    private static void setDirTurnCounDown(Integer countdown, Map<String, Object> phaseMap, CrossLightsPO crossLightsPO, Integer dir) {
        if (Objects.equals(crossLightsPO.getType(), 2)) {
            String leftTurn = String.join(Constants.SEPARATOR_UNDER_LINE, String.valueOf(dir), "1");// 圆饼灯
            phaseMap.put(String.join(Constants.SEPARATOR_UNDER_LINE, String.valueOf(dir), "2"), countdown);
            phaseMap.put(String.join(Constants.SEPARATOR_UNDER_LINE, String.valueOf(dir), "3"), countdown);
            if (!phaseMap.containsKey(leftTurn)) {
                phaseMap.put(leftTurn, countdown);
            }
        } else {
            Integer turn = crossLightsPO.getTurn();
            phaseMap.put(String.join(Constants.SEPARATOR_UNDER_LINE, String.valueOf(dir), String.valueOf(turn)), countdown);
        }
    }

    /**
     * 构建信号机请求参数
     *
     * @param signalCode
     * @return
     */
    private String getRequestBody(String signalCode) {
        RealTimeStateInfoQueryPO queryPO = new RealTimeStateInfoQueryPO();
        queryPO.setCrossCodes(Arrays.asList(signalCode));

        HKRequest request = new HKRequest();
        request.setPageNo(1);
        request.setPageSize(999);
        request.setSearchObj(queryPO);
        return JSONObject.toJSONString(request);
    }

    private Object callHKService(String apiPath, String jsonBody) {
        try {
            Map<String, String> path = PathUtil.getPathMapByApiCode(apiPath);
            String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, jsonBody, null, null, "application/json", null);
            if (StringUtils.isNotBlank(responseStr)) {
                HKResponse hkResponse = JSONObject.parseObject(responseStr, HKResponse.class);
                if (Objects.nonNull(hkResponse) && Objects.equals(Constants.HK_SUCCESS_STRING_CODE, hkResponse.getCode())) {
                    return hkResponse.getData();
                } else {
                    log.error("ArtemisHttpUtil海康服务调用异常，请求路径：{}， 返回信息： {}， 请求信息： {}", apiPath, hkResponse, jsonBody);
                }
            }
        } catch (Exception e) {
            log.error("ArtemisHttpUtil海康服务调用异常， 异常信息：", e);
        }
        return null;
    }

    @Override
    public List<LightsStatusVO> getHkLightsStatus(List<BaseCrossInfo> baseCrossInfoList) throws Exception{
        List<LightsStatusVO> result = new ArrayList<>();
        LightsStatusVO infoVo;
        try {
            Map<String, String> path = PathUtil.getPathMapByApiCode("getRealtimeSchemeStateInfo");
            for (BaseCrossInfo entity : baseCrossInfoList) {
                JSONObject jsonBody = new JSONObject();
                jsonBody.put("pageNo", 1);
                jsonBody.put("pageSize", 9999);
                JSONObject searchObj = new JSONObject();
                JSONArray crossCodes = new JSONArray();
                crossCodes.add(entity.getCode());
                searchObj.put("crossCodes", crossCodes);
                jsonBody.put("searchObj", searchObj);
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, jsonBody.toJSONString(),
                        null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);
                ManufacturerInfoPO manufacturerInfoPO =
                        manufacturerInfoMapper.selectByCode(entity.getManufacturerCode());

                String crossId = crossInfoMapper.selectIdByCodeAndManufacturerId(entity.getCode(),
                        manufacturerInfoPO.getId());
                List<CrossLightsPO> crossLightsPOS = crossLightsMapper.selectByCrossId(crossId);

                if (Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
                    JSONObject data = object.getJSONObject("data");
                    JSONArray list = data.getJSONArray("list");
                    for (int i = 0; i < list.size(); i++) {
                        infoVo = new LightsStatusVO();
                        JSONObject jsonObject = list.getJSONObject(i);
                        String patternNo = jsonObject.getString("patternNo");               // 方案编号
                        JSONArray rings = jsonObject.getJSONArray("rings");                 // 环列表
                        JSONArray channelState = jsonObject.getJSONArray("channelState");   // 灯组列表
                        String HKControlType = jsonObject.getString("controlType");
                        String WJControlType = null;
                        if (StringUtils.isNotBlank(HKControlType)) {
                            WJControlType = runModeParseMap.get(HKControlType);
                        }
                        if (StringUtils.isEmpty(WJControlType)) {
                            WJControlType = "255";// 转换失败默认临时方案
                        }
                        infoVo.setRunMode(WJControlType);// 控制模式
                        infoVo.setPhasePlanId(patternNo);                                       // 相位方案号
                        int cycleLen = jsonObject.getIntValue("cycle");                    // 周期时长
                        int curRunTime = jsonObject.getIntValue("curRunTime");             // 当前运行时长
                        int cycleCountDown = cycleLen - curRunTime;                             // 周期剩余时长
                        infoVo.setCode(entity.getCode());
                        infoVo.setCrossId(entity.getCrossId());
                        infoVo.setManufacturerCode(entity.getManufacturerCode());
                        infoVo.setCycleCountDown(cycleCountDown);
                        infoVo.setCycleLen(cycleLen);
                        // 循环相位列表
                        Map<String, Object> phaseMap = new HashMap<>();
                        for (int j = 0; j < rings.size(); j++) {
                            JSONObject jsonObj = rings.getJSONObject(j);
                            JSONArray phaseList = jsonObj.getJSONArray("phaseList");
                            for (int k = 0; k < phaseList.size(); k++) {
                                JSONObject phase = phaseList.getJSONObject(k);
                                String phaseNo = phase.getString("phaseNo");
                                Integer vehicleStatus = phase.getInteger("vehicleStatus");
                                Integer phaseLength = phase.getInteger("phaseLength");
                                Integer red = phase.getInteger("red");
                                Integer yellow = phase.getInteger("yellow");
                                Integer green = phaseLength - yellow - red;
                                phaseMap.put(phaseNo, green);
                                if (2 != vehicleStatus) { // 2红灯
                                    infoVo.setPhaseId(phaseNo);
                                }
                            }
                        }
                        infoVo.setPhaseMap(phaseMap);

                        Map<String, Object> dirLampGroupMap = new HashMap<>();
                        // 循环构建灯组对象（从灯组表获取灯组信息）
                        for (int k = 0; k < channelState.size(); k++) {
                            JSONObject channelObj = channelState.getJSONObject(k);
                            int channelNo = channelObj.getIntValue("channelNo");
                            String color = transferHKLampGroup(channelObj.getIntValue("state"));
                            for (CrossLightsPO po : crossLightsPOS) {
                                if (StringUtils.equals(po.getLightsNo(), String.valueOf(channelNo))) {
                                    String direction = po.getDir().toString();
                                    String turn = po.getTurn().toString();
                                    if (dirLampGroupMap.get(direction) != null) {
                                        Map<Integer, String> mstr = (Map<Integer, String>) dirLampGroupMap.get(direction);
                                        mstr = setTurnColor(color, po, turn, mstr);
                                        dirLampGroupMap.put(direction, mstr);
                                    } else {
                                        Map<Integer, String> lamp = setTurnColor(color, po, turn, new HashMap<>());
                                        dirLampGroupMap.put(direction, lamp);
                                    }
                                }
                            }
                        }
                        infoVo.setDirLampGroupMap(dirLampGroupMap);
                        result.add(infoVo);
                    }
                }
            }
        } catch (Exception e) {
            log.error("调用海康平台实时灯态异常：", e);
            throw new Exception(e);
        }
        return result;
    }

    private static Map<Integer, String> setTurnColor(String color, CrossLightsPO po, String turn, Map<Integer, String> mstr) {
        if (Objects.equals(2, po.getType())) {
            List<String> dirList = Arrays.asList("1", "2", "3");
            for (String commonTurn : dirList) {
                mstr.put(Integer.valueOf(commonTurn), color);
            }
        } else {
            mstr.put(Integer.valueOf(turn), color);
        }
        return mstr;
    }


    /**
     * 灯组颜色转换
     *
     * @param state
     * @return
     */
    private String transferHKLampGroup(int state) {
        String color = "";
        switch (state) {
            case 1:
                color = "green";
                break;
            case 2:
                color = "red";
                break;
            case 3:
                color = "yellow";
                break;
            case 4:
                color = "greenFlash";
                break;
            case 5:
                color = "yellowFlash";
                break;
            case 6:
                color = "red";
                break;
            case 7:
                color = "lightsOut";
                break;
            case 8:
                color = "redFlash";
                break;
            case 9:
                color = "redYellow";
                break;
            default:
                color = "";
                break;
        }
        return color;
    }
}
